/**
 * Copyright (C) 2013-2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include "IRC_Command.h"

const short NO_SET_VAL = -32768;

Jupiter::ArrayList<IRCCommand> _IRCMasterCommandList;
Jupiter::ArrayList<IRCCommand> *IRCMasterCommandList = &_IRCMasterCommandList;

IRCCommand::IRCCommand()
{
	IRCCommand::access = 0;
	IRCCommand::channelLen = 1;
	IRCCommand::channel = new char*[IRCCommand::channelLen];
	for (short i = 0; i < IRCCommand::channelLen; i++) IRCCommand::channel[i] = nullptr;
	IRCCommand::channelAccess = new short[IRCCommand::channelLen];
	for (short i = 0; i < IRCCommand::channelLen; i++) IRCCommand::channelAccess[i] = NO_SET_VAL;

	IRCCommand::typeLen = 1;
	IRCCommand::type = new short[IRCCommand::typeLen];
	for (short i = 0; i < IRCCommand::typeLen; i++) IRCCommand::type[i] = NO_SET_VAL;
	IRCCommand::typeAccess = new short[IRCCommand::typeLen];
	for (short i = 0; i < IRCCommand::typeLen; i++) IRCCommand::typeAccess[i] = NO_SET_VAL;

	IRCMasterCommandList->add(this);
}

IRCCommand::IRCCommand(const IRCCommand &command)
{
	IRCCommand::access = command.access;
	IRCCommand::channelLen = command.channelLen;
	IRCCommand::channel = new char*[IRCCommand::channelLen];
	for (short i = 0; i < IRCCommand::channelLen; i++) IRCCommand::channel[i] = command.channel[i];
	IRCCommand::channelAccess = new short[IRCCommand::channelLen];
	for (short i = 0; i < IRCCommand::channelLen; i++) IRCCommand::channelAccess[i] = command.channelAccess[i];

	IRCCommand::typeLen = command.typeLen;
	IRCCommand::type = new short[IRCCommand::typeLen];
	for (short i = 0; i < IRCCommand::typeLen; i++) IRCCommand::type[i] = command.type[i];
	IRCCommand::typeAccess = new short[IRCCommand::typeLen];
	for (short i = 0; i < IRCCommand::typeLen; i++) IRCCommand::typeAccess[i] = command.typeAccess[i];
}

IRCCommand::~IRCCommand()
{
	while (IRCCommand::channelLen > 0)
	{
		if (IRCCommand::channel[IRCCommand::channelLen - 1] != nullptr) delete[] IRCCommand::channel[IRCCommand::channelLen - 1];
		IRCCommand::channelLen--;
	}
	delete[] IRCCommand::channel;
	delete[] IRCCommand::channelAccess;
	delete[] IRCCommand::type;
	delete[] IRCCommand::typeAccess;
	for (int i = IRCMasterCommandList->size() - 1; i >= 0; i--)
	{
		if (IRCMasterCommandList->get(i) == this)
		{
			serverManager->removeCommand(this);
			IRCMasterCommandList->remove(i);
			break;
		}
	}
}

// IRC Command Functions

short IRCCommand::getAccessLevel()
{
	return IRCCommand::access;
}

short IRCCommand::getAccessLevel(short type)
{
	for (int i = 0; i < IRCCommand::typeLen; i++) if (IRCCommand::type[i] == type) return IRCCommand::typeAccess[i];
	return IRCCommand::access;
}

short IRCCommand::getAccessLevel(const char *channel)
{
	for (int i = 0; i < IRCCommand::channelLen; i++) if (IRCCommand::channel[i] != nullptr && streqli(IRCCommand::channel[i], channel)) return IRCCommand::channelAccess[i];
	return IRCCommand::access;
}

short IRCCommand::getAccessLevel(Jupiter::IRC::Client::Channel *channel)
{
	for (int i = 0; i < IRCCommand::channelLen; i++)
	{
		if (IRCCommand::channel[i] != nullptr) if (streqli(IRCCommand::channel[i], channel->getName())) return IRCCommand::channelAccess[i];
	}
	for (int i = 0; i < IRCCommand::typeLen; i++) if (IRCCommand::type[i] == channel->getType()) return IRCCommand::typeAccess[i];
	return IRCCommand::access;
}

void IRCCommand::setAccessLevel(short accessLevel)
{
	IRCCommand::access = accessLevel;
}

void IRCCommand::setAccessLevel(short type, short accessLevel)
{
	int i;
	for (i = 0; i < IRCCommand::typeLen; i++)
	{
		if (IRCCommand::type[i] == NO_SET_VAL) break;
	}
	if (i == IRCCommand::typeLen)
	{
		short tLen = IRCCommand::typeLen * 2;
		short *tType = new short[tLen];
		short *tAccess = new short[tLen];
		int b = 0;
		while (b < IRCCommand::typeLen)
		{
			tType[b] = IRCCommand::type[b];
			tAccess[b] = IRCCommand::typeAccess[b];
			b++;
		}
		b++;
		while (b < tLen)
		{
			tType[b] = NO_SET_VAL;
			tAccess[b] = NO_SET_VAL;
			b++;
		}
		delete[] IRCCommand::type;
		delete[] IRCCommand::typeAccess;
		IRCCommand::type = tType;
		IRCCommand::typeAccess = tAccess;
		IRCCommand::typeLen = tLen;
	}
	IRCCommand::type[i] = type;
	IRCCommand::typeAccess[i] = accessLevel;
}

void IRCCommand::setAccessLevel(const char *channel, short accessLevel)
{
	int i;
	for (i = 0; i < IRCCommand::channelLen; i++)
	{
		if (IRCCommand::channel[i] == nullptr) break;
	}
	if (i == IRCCommand::channelLen)
	{
		short tLen = IRCCommand::channelLen * 2;
		char **tChannel = new char*[tLen];
		short *tAccess = new short[tLen];
		int b = 0;
		while (b < IRCCommand::channelLen)
		{
			tChannel[b] = IRCCommand::channel[b];
			tAccess[b] = IRCCommand::channelAccess[b];
			b++;
		}
		b++;
		while (b < tLen)
		{
			tChannel[b] = nullptr;
			tAccess[b] = NO_SET_VAL;
			b++;
		}
		delete[] IRCCommand::channel;
		delete[] IRCCommand::channelAccess;
		IRCCommand::channel = tChannel;
		IRCCommand::channelAccess = tAccess;
		IRCCommand::channelLen = tLen;
	}
	IRCCommand::channel[i] = new char[strlen(channel) + 1];
	strcpy(IRCCommand::channel[i], channel);
	IRCCommand::channelAccess[i] = accessLevel;
}