/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include "ServerManager.h"
#include "IRC_Bot.h"
#include "IRC_Command.h"
#include "Functions.h"

ServerManager _serverManager;
ServerManager *serverManager = &_serverManager;

int ServerManager::think()
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		if (server->think() != 0) delete ServerManager::servers.remove(i);
	}
	return ServerManager::servers.size() != 0;
}

int ServerManager::addCommand(IRCCommand *command)
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		server->addCommand(command->copy());
	}
	return ServerManager::servers.size();
}

int ServerManager::removeCommand(IRCCommand *command)
{
	int r = 0;
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		if (server->freeCommand(command->getTrigger())) r++;
	}
	return r;
}

int ServerManager::removeCommand(const char *command)
{
	int r = 0;
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		if (server->freeCommand(command)) r++;
	}
	return r;
}

int ServerManager::syncCommands()
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		server->setCommandAccessLevels();
	}
	return ServerManager::servers.size();
}

IRC_Bot *ServerManager::getServer(const char *serverConfig)
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		if (streqli(server->getConfigSection(), serverConfig)) return server;
	}
	return nullptr;
}

IRC_Bot *ServerManager::getServer(unsigned int serverIndex)
{
	if (serverIndex < ServerManager::servers.size()) return ServerManager::servers.get(serverIndex);
	return nullptr;
}

bool ServerManager::addServer(const char *serverConfig)
{
	IRC_Bot *server = new IRC_Bot(serverConfig);
	if (server->connect() == true)
	{
		ServerManager::servers.add(server);
		return true;
	}
	else
	{
		delete server;
		return false;
	}
}

bool ServerManager::freeServer(unsigned int serverIndex)
{
	if (serverIndex < ServerManager::servers.size())
	{
		delete ServerManager::servers.remove(serverIndex);
		return true;
	}
	return false;
}

bool ServerManager::freeServer(IRC_Bot *server)
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		if (ServerManager::servers.get(i) == server)
		{
			delete ServerManager::servers.remove(i);
			return true;
		}
	}
	return false;
}

bool ServerManager::freeServer(const char *serverConfig)
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--)
	{
		IRC_Bot *server = ServerManager::servers.get(i);
		if (streqli(server->getConfigSection(), serverConfig))
		{
			delete ServerManager::servers.remove(i);
			return true;
		}
	}
	return false;
}

unsigned int ServerManager::size()
{
	return ServerManager::servers.size();
}

ServerManager::~ServerManager()
{
	for (int i = ServerManager::servers.size() - 1; i >= 0; i--) delete ServerManager::servers.remove(i);
}