/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include "PluginManager.h"
#include "IRC_Bot.h"
#include "Functions.h"
#include <cstring>

// Plugin Console Command
PluginConsoleCommand::PluginConsoleCommand()
{
	this->addTrigger("plugin");
	this->addTrigger("plugins");
	this->addTrigger("module");
	this->addTrigger("modules");
}

void PluginConsoleCommand::trigger(const char *parameters)
{
	if (streql(parameters, "") || strmatchi("list*", parameters))
	{
		printf("There are %u plugins loaded." ENDL, Jupiter::plugins->size());
		for (int i = Jupiter::plugins->size() - 1; i >= 0; i--) puts(Jupiter::plugins->get(i)->getName());
	}
	else
	{
		if (strmatchi("load *", parameters))
		{
			size_t t = strcspn(parameters, WHITESPACE);
			if (Jupiter::loadPlugin(parameters + t + strspn(parameters + t, " \t\r\n")) == nullptr) puts("Error: Failed to load plugin.");
			else puts("Plugin successfully loaded.");
		}
		else if (strmatchi("unload *", parameters))
		{
			size_t t = strcspn(parameters, WHITESPACE);
			const char *plugName = parameters + t + strspn(parameters + t, " \t\r\n");
			if (Jupiter::getPlugin(plugName) == nullptr) puts("Error: Plugin does not exist.");
			else if (Jupiter::freePlugin(plugName) == false) puts("Error: Failed to unload plugin.");
			else puts("Plugin successfully unloaded.");
		}
		else puts("Error: Invalid Syntax. Syntax: plugin {[list], <load> <plugin>, <unload> <plugin>}");
	}
}

const char *PluginConsoleCommand::getHelp()
{
	return "Manages plugins. Syntax: plugin {[list], <load> <plugin>, <unload> <plugin>}";
}

CONSOLE_COMMAND_INIT(PluginConsoleCommand)

// Load Plugin

void LoadPluginIRCCommand::create()
{
	this->addTrigger("loadplugin");
	this->setAccessLevel(5);
}

void LoadPluginIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters != nullptr)
	{
		if (Jupiter::getPlugin(parameters.c_str()) != nullptr) source->sendMessage(channel.c_str(), "Error: Plugin already exists. You must first unload the plugin.");
		else
		{
			if (Jupiter::loadPlugin(parameters.c_str()) == nullptr) source->sendMessage(channel.c_str(), "Error: Failed to load plugin.");
			else source->sendMessage(channel.c_str(), "Plugin successfully loaded.");
		}
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: loadPlugin <plugin>");
}

const char *LoadPluginIRCCommand::getHelp()
{
	return "Dynamically loads a plugin. Syntax: loadPlugin <plugin>";
}

IRC_COMMAND_INIT(LoadPluginIRCCommand)

// Free Plugin

void FreePluginIRCCommand::create()
{
	this->addTrigger("freeplugin");
	this->addTrigger("unloadplugin");
	this->setAccessLevel(5);
}

void FreePluginIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters != nullptr)
	{
		if (Jupiter::getPlugin(parameters.c_str()) == nullptr) source->sendMessage(channel.c_str(), "Error: Plugin does not exist.");
		else if (Jupiter::freePlugin(parameters.c_str()) == false) source->sendMessage(channel.c_str(), "Error: Failed to unload plugin.");
		else source->sendMessage(channel.c_str(), "Plugin successfully unloaded.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: freePlugin <plugin>");
}

const char *FreePluginIRCCommand::getHelp()
{
	return "Dynamically unloads a plugin. Syntax: freePlugin <plugin>";
}

IRC_COMMAND_INIT(FreePluginIRCCommand)

// List Plugins

void ListPluginIRCCommand::create()
{
	this->addTrigger("listplugin");
	this->addTrigger("listplugins");
	this->addTrigger("pluginlist");
	this->addTrigger("pluginslist");
	this->setAccessLevel(4);
}

void ListPluginIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	char msg[256];
	sprintf(msg, "%d plugins loaded:", Jupiter::plugins->size());
	source->sendMessage(channel.c_str(), msg);
	for (int i = Jupiter::plugins->size() - 1; i >= 0; i--)
	{
		source->sendMessage(channel.c_str(), Jupiter::plugins->get(i)->getName());
	}
}

const char *ListPluginIRCCommand::getHelp()
{
	return "Lists the currently loaded plugins. Syntax: pluginlist";
}

IRC_COMMAND_INIT(ListPluginIRCCommand)

// Plugin instantiation and entry point.
PluginManager pluginInstance;

extern "C" __declspec(dllexport) Jupiter::Plugin *getPlugin()
{
	return &pluginInstance;
}
