/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include "IRC_Bot.h"
#include "Functions.h"
#include "RenX_Commands.h"
#include "RenX_Core.h"
#include "RenX_Server.h"
#include "RenX_PlayerInfo.h"
#include "RenX_Functions.h"

// Console Commands

// RCON Console Command

RCONConsoleCommand::RCONConsoleCommand()
{
	this->addTrigger("rcon");
	this->addTrigger("renx");
}

void RCONConsoleCommand::trigger(const char *parameters)
{
	int i = RenX::getCore()->getServerCount();
	if (i == 0) puts("Error: Not connected to any Renegade X servers.");
	else if (parameters != nullptr)
	{
		RenX::Server *server;
		while (--i >= 0)
		{
			server = RenX::getCore()->getServer(i);
			RenX::getCore()->send(server, parameters);
		}
	}
	else puts("Error: Too Few Parameters. Syntax: rcon <input>");
}

const char *RCONConsoleCommand::getHelp()
{
	return "Sends data to the Renegade X server's rcon. Syntax: rcon <input>";
}

CONSOLE_COMMAND_INIT(RCONConsoleCommand)

// IRC Commands

// Msg IRC Command

void MsgIRCCommand::create()
{
	this->addTrigger("msg");
	this->addTrigger("say");
}

void MsgIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.size() > 0)
	{	
		Jupiter::CStringL msg = source->getChannel(channel.c_str())->getUserPrefix(nick.c_str());
		msg += nick;
		msg += "@IRC: ";
		msg += parameters;
		char *cmd = new char[msg.size() + 5];
		sprintf(cmd, "say %s", msg.c_str());
		RenX::sanitizeString(cmd);
		int r = RenX::getCore()->send(source->getChannel(channel.c_str())->getType(), cmd);
		delete[] cmd;
		if (r <= 0) source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: Msg <Message>");
}

const char *MsgIRCCommand::getHelp()
{
	return "Sends a message in-game. Syntax: Msg <Message>";
}

IRC_COMMAND_INIT(MsgIRCCommand)

// Players IRC Command

void PlayersIRCCommand::create()
{
	this->addTrigger("players");
	this->addTrigger("pl");
}

const size_t STRING_LENGTH = 240;

void PlayersIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &)
{
	short type = source->getChannel(channel.c_str())->getType();
	for (int i = RenX::getCore()->getServerCount() - 1; i >= 0; i--)
	{
		RenX::Server *server = RenX::getCore()->getServer(i);
		if (server->logChanType == type || server->adminLogChanType)
		{
			if (server->players.size() > 0)
			{
				Jupiter::DLList<char> gStrings;
				Jupiter::DLList<char> nStrings;
				Jupiter::DLList<char> oStrings;
				// Simplify later. A lot. Eventually.
				char *gCurr = nullptr;
				char *nCurr = nullptr;
				char *oCurr = nullptr;
				size_t gPos = STRING_LENGTH + 1;
				size_t nPos = STRING_LENGTH + 1;
				size_t oPos = STRING_LENGTH + 1;
				unsigned int gTotal = 0;
				unsigned int nTotal = 0;
				unsigned int oTotal = 0;
				unsigned int gBots = 0;
				unsigned int nBots = 0;
				unsigned int oBots = 0;
				for (Jupiter::DLList<RenX::PlayerInfo>::Node *node = server->players.getNode(0); node != nullptr; node = node->next)
				{
					if (node->data != nullptr)
					{
						auto name = RenX::getFormattedPlayerName(node->data);
						if (name.size() > STRING_LENGTH - 32) continue; // Name will be too long to send.

						switch (node->data->team)
						{
						case RenX::TeamType::Nod:
							if (nPos + name.size() > STRING_LENGTH)
							{
								nCurr = new char[STRING_LENGTH + 1];
								nPos = sprintf(nCurr, IRCCOLOR "04[Nod]: " IRCBOLD "%s" IRCBOLD, name.c_str());
								nStrings.add(nCurr);
							}
							else nPos += sprintf(nCurr + nPos, IRCCOLOR ", " IRCBOLD "%s" IRCBOLD, name.c_str());
							nTotal++;
							if (node->data->isBot) nBots++;
							break;
						case RenX::TeamType::GDI:
							if (gPos + name.size() > STRING_LENGTH)
							{
								gCurr = new char[STRING_LENGTH + 1];
								gPos = sprintf(gCurr, IRCCOLOR "08[GDI]: " IRCBOLD "%s" IRCBOLD, name.c_str());
								gStrings.add(gCurr);
							}
							else gPos += sprintf(gCurr + gPos, IRCCOLOR ", " IRCBOLD "%s" IRCBOLD, name.c_str());
							gTotal++;
							if (node->data->isBot) gBots++;
							break;
						default:
							if (oPos + name.size() > STRING_LENGTH)
							{
								oCurr = new char[STRING_LENGTH + 1];
								oPos = sprintf(oCurr, IRCCOLOR "14[Other]: " IRCBOLD "%s" IRCBOLD, name.c_str());
								oStrings.add(oCurr);
							}
							else oPos += sprintf(oCurr + oPos, IRCCOLOR ", " IRCBOLD "%s" IRCBOLD, name.c_str());
							oTotal++;
							if (node->data->isBot) oBots++;
							break;
						}
					}
				}
				char *outString;
				while (gStrings.size() > 0)
				{
					outString = gStrings.remove(unsigned int(0));
					source->sendMessage(channel.c_str(), outString);
					delete[] outString;
				}
				while (nStrings.size() > 0)
				{
					outString = nStrings.remove(unsigned int(0));
					source->sendMessage(channel.c_str(), outString);
					delete[] outString;
				}
				while (oStrings.size() > 0)
				{
					outString = oStrings.remove(unsigned int(0));
					source->sendMessage(channel.c_str(), outString);
					delete[] outString;
				}
				// Send totals; reuse some variables.
				outString = new char[256];
				oPos = sprintf(outString, IRCCOLOR "03Total Players" IRCCOLOR ": %u", server->players.size());
				if (gBots + nBots + oBots > 0) oPos += sprintf(outString + oPos, " (%u bots)", gBots + nBots + oBots);
				if (gTotal > 0)
				{
					oPos += sprintf(outString + oPos, IRCCOLOR "02 | " IRCCOLOR "08GDI" IRCCOLOR ": %u", gTotal);
					if (gBots > 0) oPos += sprintf(outString + oPos, " (%u bots)", gBots);
				}
				if (nTotal > 0)
				{
					oPos += sprintf(outString + oPos, IRCCOLOR "02 | " IRCCOLOR "04Nod" IRCCOLOR ": %u", nTotal);
					if (nBots > 0) oPos += sprintf(outString + oPos, " (%u bots)", nBots);
				}
				if (oTotal > 0)
				{
					oPos += sprintf(outString + oPos, IRCCOLOR "02 | " IRCCOLOR "14Other" IRCCOLOR ": %u", oTotal);
					if (oBots > 0) oPos += sprintf(outString + oPos, " (%u bots)", oBots);
				}
				source->sendMessage(channel.c_str(), outString);
				delete[] outString;
			}
			else source->sendMessage(channel.c_str(), "No players are in-game.");
		}
	}
}

const char *PlayersIRCCommand::getHelp()
{
	return "Lists the players currently in-game. Syntax: Players";
}

IRC_COMMAND_INIT(PlayersIRCCommand)

// PlayerInfo IRC Command

void PlayerInfoIRCCommand::create()
{
	this->addTrigger("playerinfo");
	this->addTrigger("pi");
	this->addTrigger("player");
	this->addTrigger("pinfo");
	this->setAccessLevel(2);
}

void PlayerInfoIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.size() > 0)
	{
		short type = source->getChannel(channel.c_str())->getType();
		RenX::PlayerInfo *player;
		Jupiter::CStringL msg;
		for (int i = RenX::getCore()->getServerCount() - 1; i >= 0; i--)
		{
			RenX::Server *server = RenX::getCore()->getServer(i);
			if ((server->logChanType == type || server->adminLogChanType) && server->players.size() > 0)
			{
				for (Jupiter::DLList<RenX::PlayerInfo>::Node *node = server->players.getNode(0); node != nullptr; node = node->next)
				{
					player = node->data;
					if (stristr(player->name.c_str(), parameters.c_str()) != nullptr)
					{
						msg.format(IRCCOLOR "03[Player Info]" IRCCOLOR "%s Name: " IRCBOLD "%s" IRCBOLD " - ID: %d - Team: " IRCBOLD "%s" IRCBOLD " - Vehicle Kills: %u - Defence Kills: %u - Building Kills: %u - Kills: %u (%u headshots) - Deaths: %u (%u suicides) - KDR: %.2f", RenX::getTeamColor(player->team), player->name.c_str(), player->id, RenX::getFullTeamName(player->team), player->vehicleKills, player->defenceKills, player->buildingKills, player->kills, player->headshots, player->deaths, player->suicides, ((float)player->kills) / (player->deaths == 0?1.0:(float)player->deaths));
						if (source->getAccessLevel(channel.c_str(), nick.c_str()) > 1)
						{
							msg += " - IP: " IRCBOLD;
							if (player->ip.size() > 0)
							{
								msg += player->ip;
								msg += IRCBOLD;
							}
							else msg += "IP Not Found" IRCBOLD;
						}
						if (player->steamid.size() > 0)
						{
							msg += " - Steam ID: " IRCBOLD;
							msg += player->steamid;
							msg += IRCBOLD;
						}
						if (player->adminType.size() > 0)
						{
							msg += " - Admin Type: " IRCBOLD;
							msg += player->adminType;
							msg += IRCBOLD;
						}
						source->sendMessage(channel.c_str(), msg.c_str());
					}
				}
			}
		}
		if (msg == "") source->sendNotice(nick.c_str(), "Error: Player not found.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: PlayerInfo <Player>");
}

const char *PlayerInfoIRCCommand::getHelp()
{
	return "Gets information about a player. Syntax: PlayerInfo <Player>";
}

IRC_COMMAND_INIT(PlayerInfoIRCCommand)

// Kill-Death Ratio IRC Command

void KillDeathRatioIRCCommand::create()
{
	this->addTrigger("kills");
	this->addTrigger("deaths");
	this->addTrigger("kdr");
	this->addTrigger("killdeathraio");
	this->setAccessLevel(2);
}

void KillDeathRatioIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.size() > 0)
	{
		short type = source->getChannel(channel.c_str())->getType();
		RenX::PlayerInfo *player;
		Jupiter::CStringL msg;
		for (int i = RenX::getCore()->getServerCount() - 1; i >= 0; i--)
		{
			RenX::Server *server = RenX::getCore()->getServer(i);
			if ((server->logChanType == type || server->adminLogChanType) && server->players.size() > 0)
			{
				for (Jupiter::DLList<RenX::PlayerInfo>::Node *node = server->players.getNode(0); node != nullptr; node = node->next)
				{
					player = node->data;
					if (stristr(player->name.c_str(), parameters.c_str()) != nullptr)
					{
						msg.format(IRCBOLD "%s" IRCBOLD IRCCOLOR ": Kills: %u - Deaths: %u - KDR: %.2f", RenX::getFormattedPlayerName(player), player->kills, player->deaths, ((float)player->kills) / (player->deaths == 0 ? 1.0 : (float)player->deaths));
						source->sendMessage(channel.c_str(), msg.c_str());
					}
				}
			}
		}
		if (msg == "") source->sendNotice(nick.c_str(), "Error: Player not found.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: Kills <Player>");
}

const char *KillDeathRatioIRCCommand::getHelp()
{
	return "Gets a player's kills and deaths. Syntax: Kills <Player>";
}

IRC_COMMAND_INIT(KillDeathRatioIRCCommand)

// ShowMods IRC Command

void ShowModsIRCCommand::create()
{
	this->addTrigger("showmods");
	this->setAccessLevel(2);
}

void ShowModsIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	int type = source->getChannel(channel.c_str())->getType();
	RenX::PlayerInfo *player;
	Jupiter::CStringL msg;
	for (int i = RenX::getCore()->getServerCount() - 1; i >= 0; i--)
	{
		RenX::Server *server = RenX::getCore()->getServer(i);
		if (server->logChanType == type || server->adminLogChanType)
		{
			msg = "say ";
			if (server->players.size() > 0)
			{
				for (Jupiter::DLList<RenX::PlayerInfo>::Node *node = server->players.getNode(0); node != nullptr; node = node->next)
				{
					player = node->data;
					if (player->adminType.size() > 0)
					{
						if (msg.size() != 4) msg += ", ";
						else msg += "Moderators in-game: ";
						msg += player->name;
					}
				}
			}
			if (msg.size() == 4) msg += "No moderators in-game.";
			server->send(msg.c_str(), msg.size());
		}
	}
	if (msg.size() == 0) source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
}

const char *ShowModsIRCCommand::getHelp()
{
	return "Sends a message, displaying the in-game moderators. Syntax: showmods";
}

IRC_COMMAND_INIT(ShowModsIRCCommand)

// Mods IRC Command

void ModsIRCCommand::create()
{
	this->addTrigger("mods");
	this->setAccessLevel(2);
}

void ModsIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.equalsi("show")) ShowModsIRCCommand_instance.trigger(source, channel, nick, parameters);
	else
	{
		int type = source->getChannel(channel.c_str())->getType();
		RenX::PlayerInfo *player;
		Jupiter::CStringL msg;
		for (int i = RenX::getCore()->getServerCount() - 1; i >= 0; i--)
		{
			RenX::Server *server = RenX::getCore()->getServer(i);
			if (server->logChanType == type || server->adminLogChanType)
			{
				msg = "";
				if (server->players.size() > 0)
				{
					for (Jupiter::DLList<RenX::PlayerInfo>::Node *node = server->players.getNode(0); node != nullptr; node = node->next)
					{
						player = node->data;
						if (player->adminType.size() > 0)
						{
							if (msg.size() != 0) msg += ", ";
							else msg += "Moderators in-game: ";
							msg += player->name;
						}
					}
				}
				if (msg.size() == 0) msg = "No moderators are in-game.";
				source->sendMessage(channel.c_str(), msg.c_str());
			}
		}
		if (msg.size() == 0) source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
}

const char *ModsIRCCommand::getHelp()
{
	return "Sends a message, displaying the in-game moderators. Syntax: mods [show]";
}

IRC_COMMAND_INIT(ModsIRCCommand)

// Reconnect IRC Command

void ReconnectIRCCommand::create()
{
	this->addTrigger("reconnect");
	this->setAccessLevel(4);
}

void ReconnectIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &)
{
	short type = source->getChannel(channel.c_str())->getType();
	Jupiter::CStringS msg;
	for (int i = RenX::getCore()->getServerCount() - 1; i >= 0; i--)
	{
		RenX::Server *server = RenX::getCore()->getServer(i);
		if (server->logChanType == type || server->adminLogChanType == type)
		{
			server->sock.closeSocket();
			if (server->connect()) msg.format("Connection established");
			else
			{
				msg.format("[RenX] ERROR: Failed to connect to %s on port %u. Error code: %d" ENDL, server->hostname, server->port, server->sock.getLastError());
				server->port = 0;
			}
			source->sendMessage(channel.c_str(), msg.c_str());
		}
	}
	if (msg == "")
	{
		// We didn't connect anywhere!!
		msg.format("ERROR: No servers found to connect to.");
		source->sendMessage(channel.c_str(), msg.c_str());
	}
}

const char *ReconnectIRCCommand::getHelp()
{
	return "Gets information about a player. Syntax: Reconnect";
}

IRC_COMMAND_INIT(ReconnectIRCCommand)

// RestartMap IRC Command

/*void RestartMapIRCCommand::create()
{
	this->addTrigger("restartmap");
	this->setAccessLevel(2);
}

void RestartMapIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	int r = RenX::getCore()->send(source->getChannel(channel.c_str())->getType(), "adminrestartmap");
	if (r > 0)
	{
		char t[256];
		sprintf(t, "Command sent to %d servers.", r);
		source->sendMessage(channel.c_str(), t);
	}
	else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
}

const char *RestartMapIRCCommand::getHelp()
{
	return "Restarts the current map. Syntax: RestartMap";
}

IRC_COMMAND_INIT(RestartMapIRCCommand)

// SetMap IRC Command

void SetMapIRCCommand::create()
{
	this->addTrigger("setmap");
	this->setAccessLevel(3);
}

void SetMapIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters != nullptr)
	{
		Jupiter::CStringL cmd = "adminrestartmap ";
		cmd += parameters;
		int r = RenX::getCore()->send(source->getChannel(channel.c_str())->getType(), cmd.c_str());
		if (r > 0)
		{
			char t[256];
			sprintf(t, "Command sent to %d servers.", r);
			source->sendMessage(channel.c_str(), t);
		}
		else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: SetMap <Map Name>");
}

const char *SetMapIRCCommand::getHelp()
{
	return "Sets the next map, and ends the current map. Syntax: SetMap <Map Name>";
}

IRC_COMMAND_INIT(SetMapIRCCommand)*/

// Mute IRC Command

/*void MuteIRCCommand::create()
{
	this->addTrigger("mute");
	this->addTrigger("silence");
	this->setAccessLevel(2);
}

void MuteIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters != nullptr)
	{
		Jupiter::CStringL cmd = "AdminForceTextMute ";
		cmd += parameters;
		int r = RenX::getCore()->send(source->getChannel(channel.c_str())->getType(), cmd.c_str());
		if (r > 0)
		{
			char t[256];
			sprintf(t, "Command sent to %d servers.", r);
			source->sendMessage(channel.c_str(), t);
		}
		else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: Mute <Player Name>");
}

const char *MuteIRCCommand::getHelp()
{
	return "Mutes a player from the game chat. Syntax: Mute <Player Name>";
}

IRC_COMMAND_INIT(MuteIRCCommand)

// UnMute IRC Command

void UnMuteIRCCommand::create()
{
	this->addTrigger("unmute");
	this->setAccessLevel(2);
}

void UnMuteIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters != nullptr)
	{
		Jupiter::CStringL cmd = "AdminForceTextUnMute ";
		cmd += parameters;
		int r = RenX::getCore()->send(source->getChannel(channel.c_str())->getType(), cmd.c_str());
		if (r > 0)
		{
			char t[256];
			sprintf(t, "Command sent to %d servers.", r);
			source->sendMessage(channel.c_str(), t);
		}
		else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: UnMute <Player Name>");
}

const char *UnMuteIRCCommand::getHelp()
{
	return "UnMutes a player from the game chat. Syntax: UnMute <Player Name>";
}

IRC_COMMAND_INIT(UnMuteIRCCommand)*/

// Kick IRC Command

void KickIRCCommand::create()
{
	this->addTrigger("kick");
	this->addTrigger("qkick");
	this->addTrigger("k");
	this->setAccessLevel(3);
}

void KickIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.size() > 0)
	{
		Jupiter::ArrayList<RenX::Server> servers = RenX::getCore()->getServers(source->getChannel(channel.c_str())->getType());
		if (servers.size() > 0)
		{
			RenX::PlayerInfo *player;
			RenX::Server *server;
			Jupiter::CStringL cmd = "adminkick ";
			unsigned int kicks = 0;
			for (int i = servers.size() - 1; i >= 0; i--)
			{
				server = servers.get(i);
				if (server != nullptr)
				{
					player = server->getPlayerByPartName(parameters);
					if (player != nullptr)
					{
						size_t len = cmd.aformat("pid%d", player->id);
						server->send(cmd.c_str(), cmd.size());
						cmd -= len;
						kicks++;
					}
				}
			}
			cmd.format("%u players kicked.", kicks);
			source->sendMessage(channel.c_str(), cmd.c_str());
		}
		else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: Kick <Player Name>");
}

const char *KickIRCCommand::getHelp()
{
	return "Kicks a player from the game. Syntax: Kick <Player Name>";
}

IRC_COMMAND_INIT(KickIRCCommand)

// KickBan IRC Command

void KickBanIRCCommand::create()
{
	this->addTrigger("kickban");
	this->addTrigger("kb");
	this->addTrigger("ban");
	this->setAccessLevel(4);
}

void KickBanIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.size() > 0)
	{
		Jupiter::ArrayList<RenX::Server> servers = RenX::getCore()->getServers(source->getChannel(channel.c_str())->getType());
		if (servers.size() > 0)
		{
			RenX::PlayerInfo *player;
			RenX::Server *server;
			Jupiter::CStringL cmd = "adminkickban ";
			unsigned int kicks = 0;
			for (int i = servers.size() - 1; i >= 0; i--)
			{
				server = servers.get(i);
				if (server != nullptr)
				{
					player = server->getPlayerByPartName(parameters);
					if (player != nullptr)
					{
						size_t len = cmd.aformat("pid%d", player->id);
						server->send(cmd.c_str(), cmd.size());
						cmd -= len;
						kicks++;
					}
				}
			}
			cmd.format("%u players kicked.", kicks);
			source->sendMessage(channel.c_str(), cmd.c_str());
		}
		else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: KickBan <Player Name>");
}

const char *KickBanIRCCommand::getHelp()
{
	return "Kicks and bans a player from the game. Syntax: KickBan <Player Name>";
}

IRC_COMMAND_INIT(KickBanIRCCommand)

// RCON IRC Command

void RCONIRCCommand::create()
{
	this->addTrigger("rcon");
	this->addTrigger("renx");
	this->setAccessLevel(5);
}

void RCONIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters != nullptr)
	{
		int r = RenX::getCore()->send(source->getChannel(channel.c_str())->getType(), parameters.c_str());
		if (r > 0)
		{
			char t[256];
			sprintf(t, "Command sent to %d servers.", r);
			source->sendMessage(channel.c_str(), t);
		}
		else source->sendMessage(channel.c_str(), "Error: Channel not attached to any connected Renegade X servers.");
	}
	else source->sendNotice(nick.c_str(), "Error: Too Few Parameters. Syntax: rcon <input>");
}

const char *RCONIRCCommand::getHelp()
{
	return "Sends data to the Renegade X server's rcon. Syntax: rcon <input>";
}

IRC_COMMAND_INIT(RCONIRCCommand)

// Game Commands

// Help Game Command

void HelpGameCommand::create()
{
	this->addTrigger("help");
}

void HelpGameCommand::trigger(RenX::Server *source, RenX::PlayerInfo *player, const Jupiter::CStringS &parameters)
{
	Jupiter::CStringL output = "say Available Commands: ";
	puts(parameters.c_str());
	if (parameters.wordCount(WHITESPACE) == 0)
	{
		int i = source->getCommandCount() - 1;
		// We know there's at least one command, because of this command!
		output += source->getCommand(i)->getTrigger();
		while (--i >= 0)
		{
			output += ", ";
			output += source->getCommand(i)->getTrigger();
		}
	}
	else
	{
		RenX::GameCommand *cmd = source->getCommand(parameters.c_str());
		if (cmd != nullptr) output += cmd->getHelp();
		else output += "Error: Command not found.";
	}
	source->send(output.c_str(), output.size());
}

const char *HelpGameCommand::getHelp()
{
	return "Lists commands, or sends command-specific help. Syntax: help [command]";
}

GAME_COMMAND_INIT(HelpGameCommand)

// Mods Game Command

void ModsGameCommand::create()
{
	this->addTrigger("mods");
	this->addTrigger("showmods");
}

void ModsGameCommand::trigger(RenX::Server *source, RenX::PlayerInfo *player, const Jupiter::CStringS &parameters)
{
	Jupiter::CStringL msg = "say ";
	for (Jupiter::DLList<RenX::PlayerInfo>::Node *node = source->players.getNode(0); node != nullptr; node = node->next)
	{
		player = node->data;
		if (player->adminType.size() > 0)
		{
			if (msg.size() != 4) msg += ", ";
			else msg += "Moderators in-game: ";
			msg += player->name;
		}
	}
	if (msg.size() == 4) msg += "No moderators are in-game.";
	source->send(msg.c_str(), msg.size());
}

const char *ModsGameCommand::getHelp()
{
	return "Displays the in-game moderators. Syntax: mods";
}

GAME_COMMAND_INIT(ModsGameCommand)

// Test Game Command

void TestGameCommand::create()
{
	this->addTrigger("test");
}

void TestGameCommand::trigger(RenX::Server *source, RenX::PlayerInfo *player, const Jupiter::CStringS &parameters)
{
	source->send("say Test!");
}

const char *TestGameCommand::getHelp()
{
	return "This is a test command!";
}

GAME_COMMAND_INIT(TestGameCommand)

// Plugin instantiation and entry point.
RenX_CommandsPlugin pluginInstance;

extern "C" __declspec(dllexport) Jupiter::Plugin *getPlugin()
{
	return &pluginInstance;
}
