/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include <ctime>
#include "RenX_Core.h"
#include "RenX_Server.h"
#include "RenX_PlayerInfo.h"
#include "RenX_Functions.h"
#include "RenX_GameCommand.h"
#include "RenX_Plugin.h"
#include "IRC_Bot.h"
#include "INIFile.h"
#include "Functions.h"

RenX::Core pluginInstance;
RenX::Core *RenXInstance = &pluginInstance;

RenX::Core *RenX::getCore()
{
	return &pluginInstance;
}

RenX::Core::Core()
{
	RenX::Server *server;

	Jupiter::CStringS serversList = Jupiter::IRC::Client::Config->get("RenX", "Servers");
	int i = serversList.wordCount(WHITESPACE);

	while (--i >= 0)
	{
		server = new RenX::Server(serversList.getWord(i, WHITESPACE));

		if (server->connect() == false)
		{
			fprintf(stderr, "[RenX] ERROR: Failed to connect to %s on port %u. Error code: %d" ENDL, server->hostname, server->port, server->sock.getLastError());
			delete server;
		}
		else pluginInstance.addServer(server);
	}
}

RenX::Core::~Core()
{
	while (RenX::Core::servers.size() > 0) delete RenX::Core::servers.remove(0);
}

int RenX::Core::send(RenX::Server *server, const char *msg)
{
	return server->send(msg);
}

int RenX::Core::send(short type, const char *msg)
{
	int r = 0;
	RenX::Server *server;
	for (int i = pluginInstance.getServerCount() - 1; i >= 0; i--)
	{
		server = pluginInstance.getServer(i);
		if ((server->logChanType == type || server->adminLogChanType == type) && pluginInstance.send(server, msg) > 0) r++;
	}
	return r;
}

void RenX::Core::addServer(RenX::Server *server)
{
	RenX::Core::servers.add(server);
}

int RenX::Core::getServerIndex(RenX::Server *server)
{
	for (int i = RenX::Core::servers.size() - 1; i >= 0; i--) if (server == RenX::Core::servers.get(i)) return i;
	return -1;
}

RenX::Server *RenX::Core::getServer(unsigned int index)
{
	return RenX::Core::servers.get(index);
}

Jupiter::ArrayList<RenX::Server> RenX::Core::getServers(int type)
{
	Jupiter::ArrayList<RenX::Server> r;
	RenX::Server *server;
	for (int i = RenX::Core::servers.size() - 1; i >= 0; i--)
	{
		server = RenX::Core::servers.get(i);
		if (server != nullptr && (server->logChanType == type || server->adminLogChanType == type)) r.add(server);
	}
	return r;
}

void RenX::Core::removeServer(unsigned int index)
{
	delete RenX::Core::servers.remove(index);
}

int RenX::Core::removeServer(RenX::Server *server)
{
	int i = RenX::Core::getServerIndex(server);
	if (i >= 0) delete RenX::Core::servers.remove(i);
	return i;
}

unsigned int RenX::Core::getServerCount()
{
	return RenX::Core::servers.size();
}

Jupiter::ArrayList<RenX::Plugin> *RenX::Core::getPlugins()
{
	return &(RenX::Core::plugins);
}

int RenX::Core::addCommand(RenX::GameCommand *command)
{
	for (int i = RenX::Core::servers.size() - 1; i >= 0; i--)
	{
		RenX::Server *server = RenX::Core::servers.get(i);
		server->addCommand(command->copy());
	}
	return RenX::Core::servers.size();
}

const char *RENX_DELIM_S = RenX::DelimS;
const char RENX_DELIM_C = RenX::DelimC;

// This shouldn't be needed later.
/*#define PARSE_PLAYER_DATA(STRING, OFFSET) \
	char *name = nullptr; int id; char team; \
	const char *p = strstr(buff, STRING); \
	if (p - buff - OFFSET < 12) { } \
	team = *(p - 3); \
	if (*(p - 9) == '<') id = atoi(p - 8); \
	else id = atoi(p - 9); \
	name = charToChar(buff + OFFSET + 1, 0, (int)(p - 10 - (buff + OFFSET + 1)));*/

#define PARSE_PLAYER_DATA() \
	Jupiter::CStringS name = playerData.gotoWord(2, ","); \
	Jupiter::CStringS idToken = playerData.getWord(1, ","); \
	int id; \
	bool isBot = false; \
	if (idToken[0] == 'b') { id = atoi(idToken.c_str() + 1); isBot = true; }\
	else id = atoi(idToken.c_str()); \
	TeamType team = RenX::getTeam(playerData.getWord(0, ",")[0]); \
	RenX::PlayerInfo *player = getPlayerOrAdd(server, name, id, team, isBot);
// Note: "id" will fail and return 0 instead if there's the 'b' bot indicator in front. Fix this later?

inline RenX::PlayerInfo *getPlayerOrAdd(RenX::Server *server, Jupiter::CStringS &name, int id, RenX::TeamType team, bool isBot)
{
	RenX::PlayerInfo *r = server->getPlayer(id);
	if (r == nullptr)
	{
		r = new RenX::PlayerInfo();
		r->id = id;
		r->name = name;
		r->isBot = isBot;
		r->joinTime = time(nullptr);
		if (id != 0) server->players.add(r);
	}
	else if (r->name.size() == 0) r->name = name;
	r->team = team;
	return r;
}

inline void onPreGameOver(RenX::Server *server, RenX::WinType winType, RenX::TeamType team, int gScore, int nScore)
{
	RenX::PlayerInfo *player;

	if (server->players.size() > 0)
	{
		for (auto *n = server->players.getNode(0); n != nullptr; n = n->next)
		{
			player = n->data;
			if (player != nullptr)
			{
				if (player->team == team) player->wins++;
				else player->loses++;
			}
		}
	}
}

inline void onPostGameOver(RenX::Server *server, RenX::WinType winType, RenX::TeamType team, int gScore, int nScore)
{
	RenX::PlayerInfo *player;

	if (server->players.size() > 0)
	{
		for (auto *n = server->players.getNode(0); n != nullptr; n = n->next)
		{
			player = n->data;
			if (player != nullptr)
			{
				player->kills = 0;
				player->deaths = 0;
				player->suicides = 0;
				player->headshots = 0;
				player->vehicleKills = 0;
				player->buildingKills = 0;
				player->defenceKills = 0;
			}
		}
	}

	//server->send("logclientlist");
}

inline void onChat(RenX::Server *server, RenX::PlayerInfo *player, const Jupiter::CStringS &message, bool isPublic)
{
	const Jupiter::CStringS &prefix = server->getCommandPrefix();
	if (prefix == "" || stristr(message.c_str(), prefix.c_str()) == message.c_str())
	{
		Jupiter::CStringS command;
		Jupiter::CStringS parameters;
		if (containsSymbol(WHITESPACE, *(message.c_str() + prefix.size())))
		{
			command = message.getWord(1, WHITESPACE);
			parameters = message.gotoWord(2, WHITESPACE);
		}
		else
		{
			command.setString(getWord(message.c_str() + prefix.size(), 0));
			parameters = message.gotoWord(1, WHITESPACE);
		}
		server->triggerCommand(command.c_str(), player, parameters);
	}
}

int RenX::Core::think()
{
	// Change this later to just call server->think().
	RenX::Server *server;
	int a = RenX::Core::servers.size() - 1;
	while (a >= 0)
	{
		server = RenX::Core::servers.get(a);
		if (server->port == 0) delete RenX::Core::servers.remove(a);
		else
		{
			if (server->sock.recv() > 0)
			{
				Jupiter::CStringS buffer = server->sock.getBuffer();
				int totalLines = countLines(buffer.c_str());
				for (int currentLine = 0; currentLine < totalLines; currentLine++)
				{
					Jupiter::CStringS buff = buffer.getWord(currentLine, ENDL);
					Jupiter::CStringS header = buff.getWord(0, RENX_DELIM_S);
					Jupiter::CStringS playerData = buff.getWord(1, RENX_DELIM_S);
					Jupiter::CStringS action = buff.getWord(2, RENX_DELIM_S);

					// Debugging stuffs that would spam normal people's consoles.
					//printf("[RenX Dump] Buffer: %s" ENDL, buff.c_str());
					//printf("[RenX Dump] %s - %s - %s" ENDL, header.c_str(), playerData.c_str(), action.c_str());

					if (buff.size() > 0)
					{
						switch (header[0])
						{
						case 'l':
							if (header.equals("lGAME:"))
							{
								if (action.equals("deployed"))
								{
									PARSE_PLAYER_DATA();
									Jupiter::CStringS objectType = buff.getWord(3, RENX_DELIM_S);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnDeploy(server, player, objectType);
								}
								else if (action.equals("suicided by"))
								{
									PARSE_PLAYER_DATA();
									player->deaths++;
									player->suicides++;
									Jupiter::CStringS damageType = buff.getWord(3, RENX_DELIM_S);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnSuicide(server, player, damageType);
								}
								else if (action.equals("killed"))
								{
									PARSE_PLAYER_DATA();
									Jupiter::CStringS victimData = buff.getWord(3, RENX_DELIM_S);
									Jupiter::CStringS vname = victimData.getWord(2, ",");
									Jupiter::CStringS vidToken = victimData.getWord(1, ",");
									int vid;
									bool visBot = false;
									if (vidToken[0] == 'b')
									{
										vid = atoi(vidToken.c_str() + 1);
										visBot = true;
									}
									else vid = atoi(vidToken.c_str());
									TeamType vteam = RenX::getTeam(victimData.getWord(0, ",")[0]);
									Jupiter::CStringS damageType = buff.getWord(5, RENX_DELIM_S);
									RenX::PlayerInfo *victim = getPlayerOrAdd(server, vname, vid, vteam, visBot);
									player->kills++;
									if (damageType.equals("Rx_DmgType_Headshot")) player->headshots++;
									victim->deaths++;

									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnKill(server, player, victim, damageType);
								}
								else if (action.match("died by"))
								{
									PARSE_PLAYER_DATA();
									player->deaths++;
									Jupiter::CStringS damageType = buff.getWord(3, RENX_DELIM_S);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnDie(server, player, damageType);
								}
								else if (action.match("destroyed*"))
								{
									PARSE_PLAYER_DATA();
									Jupiter::CStringS victim = buff.getWord(3, RENX_DELIM_S);
									Jupiter::CStringS damageType = buff.getWord(5, RENX_DELIM_S);
									ObjectType type;
									if (action.equals("destroyed building"))
									{
										type = Building;
										player->buildingKills++;
									}
									else if (victim.match("Rx_Defence_*"))
									{
										type = Defence;
										player->defenceKills++;
									}
									else
									{
										type = Vehicle;
										player->vehicleKills++;
									}
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnDestroy(server, player, victim, damageType, type);
								}
								else if (playerData.match("??? wins (*)"))
								{
									TeamType team = RenX::getTeam(playerData[0]);
									int gScore = atoi(buff.getWord(2, RENX_DELIM_S).c_str() + 4);
									int nScore = atoi(buff.getWord(3, RENX_DELIM_S).c_str() + 4);
									Jupiter::CStringS winType = playerData.c_str() + 10;
									winType -= 1;
									WinType iWinType = Unknown;
									if (gScore == nScore) iWinType = Tie;
									else if (winType.equals("TimeLimit")) iWinType = Score;
									else if (winType.equals("Buildings")) iWinType = Base;
									onPreGameOver(server, iWinType, team, gScore, nScore);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnGameOver(server, iWinType, team, gScore, nScore);
									onPostGameOver(server, iWinType, team, gScore, nScore);
								}
								/*else if (action.match("wins *"))
								{
									TeamType team = RenX::getTeam(playerData[0]);
									int gScore = atoi(buff.getWord(3, RENX_DELIM_S).c_str() + 4);
									int nScore = atoi(buff.getWord(4, RENX_DELIM_S).c_str() + 4);
									Jupiter::CStringS winType = action.c_str() + 6;
									winType -= 1;
									WinType iWinType = Unknown;
									if (gScore == nScore) iWinType = Tie;
									else if (winType.equals("TimeLimit")) iWinType = Score;
									else if (winType.equals("Buildings")) iWinType = Base;
									onGameOver(server, iWinType, team, gScore, nScore);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnGameOver(server, iWinType, team, gScore, nScore);
								}*/
								else if (playerData.equals("Tie"))
								{
									int gScore = atoi(action.c_str() + 4);
									int nScore = atoi(buff.getWord(3, RENX_DELIM_S).c_str() + 4);

									if (gScore == nScore)
									{
										onPreGameOver(server, Tie, Other, gScore, nScore);
										for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnGameOver(server, Tie, Other, gScore, nScore);
										onPostGameOver(server, Tie, Other, gScore, nScore);
									}
									else
									{
										TeamType winTeam = gScore > nScore ? RenX::getTeam('G') : RenX::getTeam('N');
										onPreGameOver(server, Shutdown, winTeam, gScore, nScore);
										for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnGameOver(server, Shutdown, winTeam, gScore, nScore);
										onPostGameOver(server, Shutdown, winTeam, gScore, nScore);
									}
								}
								else for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnGame(server, buff.gotoWord(1, RENX_DELIM_S));
							}
							else if (header.equals("lCHAT:"))
							{
								if (action.equals("teamsay:"))
								{
									PARSE_PLAYER_DATA();
									Jupiter::CStringS message = buff.getWord(3, RENX_DELIM_S);
									onChat(server, player, message, false);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnTeamChat(server, player, message);
								}
								else if (action.equals("say:"))
								{
									PARSE_PLAYER_DATA();
									Jupiter::CStringS message = buff.getWord(3, RENX_DELIM_S);
									onChat(server, player, message, true);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnChat(server, player, message);
								}
							}
							else if (header.equals("lPLAYER:"))
							{
								PARSE_PLAYER_DATA();
								if (action.equals("disconnected"))
								{
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnPart(server, player);
									server->removePlayer(player);
									player = nullptr;
								}
								else if (action.equals("entered from"))
								{
									player->ip = buff.getWord(3, RENX_DELIM_S);
									if (buff.getWord(4, RENX_DELIM_S).equals("steamid")) player->steamid = buff.getWord(5, RENX_DELIM_S);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnJoin(server, player);
								}
								else if (action.equals("changed name to:"))
								{
									Jupiter::CStringS newName = buff.getWord(3, RENX_DELIM_S);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnNameChange(server, player, newName);
									player->name = newName;
								}
							}
							else if (header.equals("lRCON:"))
							{
								if (action.equals("executed:"))
								{
									Jupiter::CStringS command = buff.getWord(3, RenX::DelimS);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnExecute(server, playerData, command);
								}
								else if (action.equals("subscribed")) for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnSubscribe(server, playerData);
								else for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnRCON(server, buff.gotoWord(1, RENX_DELIM_S));
							}
							else if (header.equals("lADMIN:"))
							{
								PARSE_PLAYER_DATA();
								if (action.equals("logged in as"))
								{
									player->adminType = buff.getWord(3, RenX::DelimS);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnAdminLogin(server, player);
								}
								else if (action.equals("logged out of"))
								{
									Jupiter::CStringS type = buff.getWord(3, RenX::DelimS);
									for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnAdminLogout(server, player);
									player->adminType = "";
								}
								else for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnAdmin(server, buff.gotoWord(1, RENX_DELIM_S));
							}
							else if (header.equals("lC-LIST:"))
							{
								// ID IP SteamID Team Name
								
								int id;
								bool isBot = false;
								if (playerData.c_str()[0] == 'b')
								{
									id = atoi(playerData.c_str() + 1);
									isBot = true;
								}
								else id = atoi(playerData.c_str());
								Jupiter::CStringS ip = playerData.getWord(1, WHITESPACE);
								Jupiter::CStringS steamid = playerData.getWord(2, WHITESPACE);
								RenX::TeamType team;
								Jupiter::CStringS name;
								if (steamid.equals("-----NO"))
								{
									steamid = "";
									team = getTeam(playerData.getWord(4, WHITESPACE)[0]);
									name = playerData.gotoWord(5, WHITESPACE);
								}
								else
								{
									team = getTeam(playerData.getWord(3, WHITESPACE)[0]);
									name = playerData.gotoWord(4, WHITESPACE);
								}

								RenX::PlayerInfo *player = getPlayerOrAdd(server, name, id, team, isBot);
								player->ip = ip;
								player->steamid = steamid;
							}
							else for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnLog(server, buff.c_str()+1);
							break;
						case 'c':
						{
							Jupiter::CStringS raw = buff.c_str() + 1;
							for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnCommand(server, raw);
						}
							break;
						case 'e':
						{
							Jupiter::CStringS raw = buff.c_str() + 1;
							for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnError(server, raw);
						}
							break;
						case 'v':
						{
							Jupiter::CStringS raw = buff.c_str() + 1;
							for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnVersion(server, raw);
						}
							break;
						case 'a':
						{
							Jupiter::CStringS raw = buff.c_str() + 1;
							for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnAuthorized(server, raw);
						}
							break;
						default:
						{
							Jupiter::CStringS raw = buff.c_str() + 1;
							for (int i = RenX::Core::plugins.size() - 1; i >= 0; i--) RenX::Core::plugins.get(i)->RenX_OnOther(server, buff[0], raw);
						}
							break;
						}
					}
				}
			}
			else if (server->sock.getLastError() != 10035) // This is a serious error
			{
				// Really shitty reconnect mechanism. Make this more closely resemble the IRC client reconnection setup later.
				server->sock.closeSocket();
				if (server->connect() == false) RenX::Core::removeServer(server);
			}
		}
		a--;
	}
	return Jupiter::Plugin::think();
}

// Entry point

extern "C" __declspec(dllexport) Jupiter::Plugin *getPlugin()
{
	return &pluginInstance;
}

// Unload

extern "C" __declspec(dllexport) void unload(void)
{
	while (pluginInstance.getPlugins()->size() > 0) freePlugin(pluginInstance.getPlugins()->remove(0));
}
