/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include <ctime>
#include <cstring>
#include "Functions.h"
#include "IRC_Bot.h"
#include "ServerManager.h"
#include "RenX_Functions.h"
#include "RenX_Core.h"
#include "RenX_Server.h"
#include "RenX_PlayerInfo.h"

const char *NodColor = "04";
const char *GDIColor = "08";
const char *OtherColor = "14";
const char *NodShortName = "Nod";
const char *GDIShortName = "GDI";
const char *OtherShortName = "N/A";
const char *NodLongName = "Brotherhood of Nod";
const char *GDILongName = "Global Defense Initiative";
const char *OtherLongName = "Unknown";
const char *RenX::DelimS = "\xA0";
const char RenX::DelimC = *RenX::DelimS;

RenX::TeamType RenX::getTeam(char team)
{
	switch (team)
	{
	case 'g':
	case 'G':
		return GDI;
	case 'n':
	case 'N':
		return Nod;
	default:
		return Other;
	}
}

const char *RenX::getTeamColor(TeamType team)
{
	switch (team)
	{
	case GDI:
		return GDIColor;
	case Nod:
		return NodColor;
	default:
		return OtherColor;
	}
}

const char *RenX::getTeamName(TeamType team)
{
	switch (team)
	{
	case GDI:
		return GDIShortName;
	case Nod:
		return NodShortName;
	default:
		return OtherShortName;
	}
}

const char *RenX::getFullTeamName(TeamType team)
{
	switch (team)
	{
	case GDI:
		return GDILongName;
	case Nod:
		return NodLongName;
	default:
		return OtherLongName;
	}
}

RenX::TeamType RenX::getEnemy(TeamType team)
{
	switch (team)
	{
	case GDI:
		return Nod;
	case Nod:
		return GDI;
	default:
		return Other;
	}
}

const char *RenX::translateName(const char *object)
{
	if (object == nullptr) return "";
	if (strstr(object, "Rx_") == object) object += 3;
	if (strstr(object, "Vehicle_") == object)
	{
		object += 8;

		/** Nod Vehicles */
		if (streql(object, "Harvester_Nod")) return "Nod Harvester";
		// Buggy -> Buggy
		if (streql(object, "Artillery")) return "Mobile Artillery";
		if (streql(object, "APC_Nod")) return "Nod APC";
		if (streql(object, "LightTank")) return "Light Tank";
		if (streql(object, "FlameTank")) return "Flame Tank";
		if (streql(object, "StealthTank")) return "Stealth Tank";
		if (streql(object, "Chinook_Nod")) return "Nod Chinook";
		// Apache -> Apache

		/** GDI Vehicles */
		if (streql(object, "Harvester_GDI")) return "GDI Harvester";
		// Humvee -> Humvee
		if (streql(object, "MRLS")) return "Mobile Rocket Launcher System";
		if (streql(object, "APC_GDI")) return "GDI APC";
		if (streql(object, "MediumTank")) return "Medium Tank";
		if (streql(object, "MammothTank")) return "Mammoth Tank";
		if (streql(object, "Chinook_GDI")) return "GDI Chinook";
		// Orca -> Orca

		/** Other Vehicles */
		if (streql(object, "A10_DmgType_GattlingGun")) return "A10 Thunderbolt Gattling Gun";
		if (streql(object, "A10_DmgType_Bomb")) return "A10 Thunderbolt Bomb";
	}
	else if (strstr(object, "DmgType_") == object)
	{
		object += 8;

		/** Non-weapon damage types */
		if (streql(object, "Suicided")) return "Suicide";
		if (streql(object, "Tiberium")) return "Tiberium";
		if (streql(object, "TiberiumBleed")) return "Tiberium Decay";
		if (streql(object, "RanOver")) return "Crushed";

		/** Infantry weapons */
		// Pistol -> Pistol
		if (streql(object, "SMG")) return "Machine Pistol";
		if (streql(object, "HeavyPistol")) return "Heavy Pistol";
		// Carbine -> Carbine
		if (streql(object, "TiberiumFlechetteRifle")) return "Tiberium Flechette Gun"; // Not a rifle.
		if (streql(object, "TiberiumAutoRifle")) return "Tiberium Automatic Rifle";

		// Grenade -> Grenade
		if (streql(object, "TimedC4")) return "Timed C4";
		if (streql(object, "RemoteC4")) return "Remote C4";
		if (streql(object, "ProxyC4")) return "Proximity C4";
		if (streql(object, "BurnC4")) return "C4 Burn";
		if (streql(object, "FireBleed")) return "Fire Burn"; // Caused by C4 or flame weapons

		if (streql(object, "AutoRifle")) return "Automatic Rifle";
		// Shotgun -> Shotgun
		if (streql(object, "FlameThrower")) return "Flamethrower";
		if (streql(object, "GrenadeLauncher")) return "Grenade Launcher";
		if (streql(object, "MarksmanRifle")) return "Marksman's Rifle";

		if (streql(object, "ChainGun")) return "Chain Gun";
		if (streql(object, "MissileLauncher")) return "Missile Launcher";
		if (streql(object, "MissileLauncher_Alt")) return "Missile Launcher";
		if (streql(object, "ChemicalThrower")) return "Chemical Spray Gun";
		if (streql(object, "LaserRifle")) return "Laser Rifle";
		if (streql(object, "TacticalRifle")) return "Tactical Rifle";
		if (streql(object, "RocketLauncher")) return "Rocket Launcher";
		if (streql(object, "LaserChainGun")) return "Laser Chain Gun";
		if (streql(object, "FlakCannon")) return "Flak";
		if (streql(object, "FlakCannon_Alt")) return "Concentrated Flak";
		if (streql(object, "SniperRifle")) return "Sniper Rifle";
		if (streql(object, "RamjetRifle")) return "Ramjet";
		// Railgun -> Railgun
		if (streql(object, "PersonalIonCannon")) return "Personal Ion Cannon";
		if (streql(object, "VoltRifle")) return "Volt Rifle";
		if (streql(object, "VoltRifle_Alt")) return "Volt Rifle Burst";

		/** Vehicle weapons */
		if (streql(object, "MammothTank_Missile")) return "Mammoth Tank Missile";
		if (streql(object, "MammothTank_Cannon")) return "Mammoth Tank Cannon";
		if (streql(object, "Orca_Missile")) return "Orca Missle";
		if (streql(object, "Orca_Gun")) return "Orca Gun";
		if (streql(object, "Apache_Rocket")) return "Apache Rocket";
		if (streql(object, "Apache_Gun")) return "Apache Gun";

		/** Other weapons */
		if (streql(object, "Nuke")) return "Nuclear Missile Strike";
		if (streql(object, "IonCannon")) return "Ion Cannon Strike";
		if (streql(object, "GunEmpl")) return "Gun Emplacement Gattling Gun";
		if (streql(object, "GunEmpl_Alt")) return "Gun Emplacement Automatic Cannon";
		if (streql(object, "RocketEmpl_Swarm")) return "Rocket Emplacement Swarm Missile";
		if (streql(object, "RocketEmpl_Missile")) return "Rocket Emplacement Hellfire Missile";

		// Vehicles copied from above.

		/** Nod Vehicles */
		if (streql(object, "Harvester_Nod")) return "Nod Harvester";
		// Buggy -> Buggy
		if (streql(object, "Artillery")) return "Mobile Artillery";
		if (streql(object, "APC_Nod")) return "Nod APC";
		if (streql(object, "LightTank")) return "Light Tank";
		if (streql(object, "FlameTank")) return "Flame Tank";
		if (streql(object, "StealthTank")) return "Stealth Tank";
		if (streql(object, "Chinook_Nod")) return "Nod Chinook";
		// Apache -> Apache

		/** GDI Vehicles */
		if (streql(object, "Harvester_GDI")) return "GDI Harvester";
		// Humvee -> Humvee
		if (streql(object, "MRLS")) return "Mobile Rocket Launcher System";
		if (streql(object, "APC_GDI")) return "GDI APC";
		if (streql(object, "MediumTank")) return "Medium Tank";
		if (streql(object, "MammothTank")) return "Mammoth Tank";
		if (streql(object, "Chinook_GDI")) return "GDI Chinook";
		// Orca -> Orca

		/** Other Vehicles */
		if (streql(object, "A10_DmgType_GattlingGun")) return "A10 Thunderbolt Gattling Gun";
		if (streql(object, "A10_DmgType_Bomb")) return "A10 Thunderbolt Bomb";
	}
	else if (strstr(object, "Building_") == object)
	{
		object += 9;
		/** Nod structures */
		if (streql(object, "HandOfNod_Internals")) return "Hand of Nod";
		if (streql(object, "AirTower_Internals")) return "Airstrip";
		if (streql(object, "Refinery_Nod_Internals")) return "Nod Refinery";
		if (streql(object, "PowerPlant_Nod_Internals")) return "Nod Power Plant";
		
		/** GDI structures */
		if (streql(object, "Barracks_Internals")) return "Barracks";
		if (streql(object, "WeaponsFactory_Internals")) return "Weapons Factory";
		if (streql(object, "Refinery_GDI_Internals")) return "GDI Refinery";
		if (streql(object, "PowerPlant_GDI_Internals")) return "GDI Power Plant";

		/** Defense structures */
		if (streql(object, "AdvancedGuardTower_Internals")) return "Advanced Guard Tower";
		if (streql(object, "Obelisk_Internals")) return "Obelisk of Light";
	}
	else if (strstr(object, "Defence_") == object)
	{
		object += 8;
		if (streql(object, "GuardTower")) return "Guard Tower";
		if (streql(object, "Turret")) return "Turret";
		if (streql(object, "SAMSite")) return "SAM Site";
		if (streql(object, "AATower")) return "Anti-Air Guard Tower";
		if (streql(object, "GunEmplacement")) return "Gun Emplacement";
		if (streql(object, "RocketEmplacement")) return "Rocket Emplacement";
	}
	else if (strstr(object, "UTDmgType_") == object)
	{
		object += 10;
		if (streql(object, "VehicleExplosion")) return "Vehicle Explosion";
		// Drowned -> Drowned
	}
	else if (streql(object, "IonCannonBeacon")) return "Ion Cannon Strike";
	else if (streql(object, "NukeBeacon")) return "Nuclear Strike Beacon";
	return object;
}

const char *RenX::translateWinType(RenX::WinType winType)
{
	switch (winType)
	{
	case Score:
		return "Domination (High Score)";
	case Base:
		return "Conquest (Base Destruction)";
	case Beacon:
		return "Espionage (Beacon)";
	case Tie:
		return "Draw (Tie)";
	case Shutdown:
		return "Ceasefire (Shutdown)";
	case Unknown:
	default:
		return "Aliens (Unknown)";
	}
}

const char *RenX::translateWinTypePlain(RenX::WinType winType)
{
	switch (winType)
	{
	case Score:
		return "High Score";
	case Base:
		return "Base Destruction";
	case Beacon:
		return "Beacon";
	case Tie:
		return "Tie";
	case Shutdown:
		return "Shutdown";
	case Unknown:
	default:
		return "Unknown";
	}
}

Jupiter::CStringL RenX::getFormattedPlayerName(const RenX::PlayerInfo *player)
{
	Jupiter::CStringL r;
	if (player->isBot) r += IRCCOLOR "05[B]";
	r += IRCCOLOR;
	r += RenX::getTeamColor(player->team);
	r += player->name;
	return r;
}

void RenX::sanitizeString(char *str)
{
	while (*str != 0)
	{
		switch (*str)
		{
		case '|':
			*str = '/';
			break;
		case '\\':
			if (*(str + 1) == 0) *str = '/'; // So users get something at least.
			break;
		default:
			break;
		}
		str++;
	}
}

time_t RenX::getGameTime(const RenX::PlayerInfo *player)
{
	time_t currentTime = time(0);
	if (player->joinTime == currentTime) currentTime++;
	return currentTime - player->joinTime;
}

double RenX::getKillDeathRatio(const RenX::PlayerInfo *player, bool includeSuicides)
{
	double deaths = player->deaths;
	if (includeSuicides == false) deaths -= player->suicides;
	if (deaths == 0) deaths = 1;
	return ((double)player->kills) / deaths;
}

double RenX::getHeadshotKillRatio(const RenX::PlayerInfo *player)
{
	if (player->kills == 0) return 0;
	return ((double)player->headshots) / ((double)player->kills);
}

double RenX::getKillsPerSecond(const RenX::PlayerInfo *player)
{
	return ((double)player->kills) / ((double)RenX::getGameTime(player));
}