/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include <ctime>
#include "ExcessiveHeadshots.h"
#include "IRC_Client.h"
#include "INIFile.h"
#include "RenX_Server.h"
#include "RenX_PlayerInfo.h"
#include "RenX_Functions.h"

RenX_ExcessiveHeadshotsPlugin::RenX_ExcessiveHeadshotsPlugin()
{
	const char *t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "HeadshotKillRatio");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::ratio = atof(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "Kills");
	if (t != nullptr)
	{
		int tKills = atoi(t);
		if (tKills < 0) fprintf(stderr, "[RenX.ExcessiveHeadshots] Warning: \"%d\" is an invalid input for \"Kills\". Value MUST be positive. Defaulting to %u...", tKills, RenX_ExcessiveHeadshotsPlugin::minKills);
		else RenX_ExcessiveHeadshotsPlugin::minKills = tKills;
	}

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "KillDeathRatio");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::minKD = atof(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "KillsPerSecond");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::minKPS = atof(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "Flags");
	if (t != nullptr)
	{
		int tFlags = atoi(t);
		if (tFlags < 2) fprintf(stderr, "[RenX.ExcessiveHeadshots] Warning: \"%d\" is an invalid input for \"Flags\". Value MUST be AT LEAST 2. Defaulting to %u...", tFlags, RenX_ExcessiveHeadshotsPlugin::minFlags);
		else RenX_ExcessiveHeadshotsPlugin::minFlags = tFlags;
	}
}

void RenX_ExcessiveHeadshotsPlugin::OnRehash(Jupiter::IRC::Client *server)
{
	const char *t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "HeadshotKillRatio");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::ratio = atof(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "Kills");
	if (t != nullptr)
	{
		int tKills = atoi(t);
		if (tKills < 0) fprintf(stderr, "[RenX.ExcessiveHeadshots] Warning: \"%d\" is an invalid input for \"Kills\". Value MUST be positive. Defaulting to %u...", tKills, RenX_ExcessiveHeadshotsPlugin::minKills);
		else RenX_ExcessiveHeadshotsPlugin::minKills = tKills;
	}

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "KillDeathRatio");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::minKD = atof(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "KillsPerSecond");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::minKPS = atof(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "MaxGameTime");
	if (t != nullptr) RenX_ExcessiveHeadshotsPlugin::maxGameTime = atoi(t);

	t = Jupiter::IRC::Client::Config->get(RenX_ExcessiveHeadshotsPlugin::getName(), "Flags");
	if (t != nullptr)
	{
		int tFlags = atoi(t);
		if (tFlags < 2) fprintf(stderr, "[RenX.ExcessiveHeadshots] Warning: \"%d\" is an invalid input for \"Flags\". Value MUST be AT LEAST 2. Defaulting to %u...", tFlags, RenX_ExcessiveHeadshotsPlugin::minFlags);
		else RenX_ExcessiveHeadshotsPlugin::minFlags = tFlags;
	}
}

void RenX_ExcessiveHeadshotsPlugin::RenX_OnKill(RenX::Server *server, const RenX::PlayerInfo *player, const RenX::PlayerInfo *victim, const Jupiter::CStringS &damageType)
{
	if (damageType.equals("Rx_DmgType_Headshot"))
	{
		unsigned int flags = 0;
		if (player->kills >= RenX_ExcessiveHeadshotsPlugin::minKills) flags++;
		if (RenX::getHeadshotKillRatio(player) >= RenX_ExcessiveHeadshotsPlugin::ratio) flags++;
		if (RenX::getKillDeathRatio(player) >= RenX_ExcessiveHeadshotsPlugin::minKD) flags++;
		if (RenX::getKillsPerSecond(player) >= RenX_ExcessiveHeadshotsPlugin::minKPS) flags++;
		if (RenX::getKillsPerSecond(player) >= RenX_ExcessiveHeadshotsPlugin::minKPS * 2) flags++;
		if (RenX::getGameTime(player) <= RenX_ExcessiveHeadshotsPlugin::maxGameTime) flags++;

		if (flags >= RenX_ExcessiveHeadshotsPlugin::minFlags)
		{
			Jupiter::CStringL cmd = Jupiter::CStringL::Format("adminkickban pid%d", player->id);
			server->send(cmd.c_str(), cmd.size());
			server->sendPubChan(IRCCOLOR "13[Aimbot]" IRCCOLOR " %s was banned from the server! Kills: %u - Deaths: %u - Headshots: %u", player->name.c_str(), player->kills, player->deaths, player->headshots);
			server->sendAdmChan(IRCCOLOR "13[Aimbot]" IRCCOLOR " %s was banned from the server! Kills: %u - Deaths: %u - Headshots: %u - IP: " IRCBOLD "%s" IRCBOLD " - Steam ID: " IRCBOLD "%s" IRCBOLD, player->name.c_str(), player->kills, player->deaths, player->headshots, player->ip.c_str(), player->steamid.c_str());
		}
	}
}


// Plugin instantiation and entry point.
RenX_ExcessiveHeadshotsPlugin pluginInstance;

extern "C" __declspec(dllexport) Jupiter::Plugin *getPlugin()
{
	return &pluginInstance;
}
