/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include "RenX_Logging.h"
#include "RenX_Core.h"
#include "RenX_PlayerInfo.h"
#include "RenX_Functions.h"
#include "RenX_Server.h"
#include "IRC_Bot.h"

void RenX_LoggingPlugin::RenX_OnJoin(RenX::Server *server, const RenX::PlayerInfo *player)
{
	server->sendPubChan(IRCCOLOR "12[Join]" IRCCOLOR "%s " IRCBOLD "%s" IRCBOLD " joined the game fighting for the %s!", RenX::getTeamColor(player->team), player->name.c_str(), RenX::getFullTeamName(player->team));
	if (player->steamid == nullptr) server->sendAdmChan(IRCCOLOR "12[Join]" IRCCOLOR "%s " IRCBOLD "%s" IRCBOLD " joined the game fighting for the %s from " IRCBOLD "%s" IRCBOLD ", but is not using Steam.", RenX::getTeamColor(player->team), player->name.c_str(), RenX::getFullTeamName(player->team), player->ip.c_str());
	else server->sendAdmChan(IRCCOLOR "12[Join]" IRCCOLOR "%s " IRCBOLD "%s" IRCBOLD " joined the game fighting for the %s from " IRCBOLD "%s" IRCBOLD " using Steam ID " IRCBOLD "%s" IRCBOLD ".", RenX::getTeamColor(player->team), player->name.c_str(), RenX::getFullTeamName(player->team), player->ip.c_str(), player->steamid.c_str());
}

void RenX_LoggingPlugin::RenX_OnPart(RenX::Server *server, const RenX::PlayerInfo *player)
{
	server->sendLogChan(IRCCOLOR "12[Part] " IRCBOLD "%s" IRCBOLD " has left the %s.", RenX::getFormattedPlayerName(player).c_str(), RenX::getFullTeamName(player->team));
}

void RenX_LoggingPlugin::RenX_OnNameChange(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &newPlayerName)
{
	server->sendLogChan(IRCBOLD "%s" IRCBOLD " has changed their name to " IRCBOLD "%s" IRCBOLD ".", RenX::getFormattedPlayerName(player).c_str(), newPlayerName.c_str());
}

void RenX_LoggingPlugin::RenX_OnChat(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &message)
{
	server->sendLogChan(IRCBOLD "%s" IRCCOLOR IRCBOLD ": %s", RenX::getFormattedPlayerName(player).c_str(), message.c_str());
}

void RenX_LoggingPlugin::RenX_OnTeamChat(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &message)
{
	server->sendLogChan(IRCBOLD "%s" IRCBOLD ": %s", RenX::getFormattedPlayerName(player).c_str(), message.c_str());
}

void RenX_LoggingPlugin::RenX_OnDeploy(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &object)
{
	server->sendLogChan(IRCBOLD "%s" IRCBOLD " deployed a " IRCBOLD "%s" IRCBOLD, RenX::getFormattedPlayerName(player).c_str(), RenX::translateName(object.c_str()));
}

void RenX_LoggingPlugin::RenX_OnSuicide(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &message)
{
	server->sendLogChan("%s" IRCCOLOR " suicided (death by " IRCCOLOR "12%s" IRCCOLOR ").", RenX::getFormattedPlayerName(player).c_str(), RenX::translateName(message.c_str()));
}

void RenX_LoggingPlugin::RenX_OnKill(RenX::Server *server, const RenX::PlayerInfo *player, const RenX::PlayerInfo *victim, const Jupiter::CStringS &damageType)
{
	server->sendLogChan("%s" IRCCOLOR " killed %s" IRCCOLOR " (" IRCCOLOR "12%s" IRCCOLOR ").", RenX::getFormattedPlayerName(player).c_str(), RenX::getFormattedPlayerName(victim).c_str(), RenX::translateName(damageType.c_str()));
}

void RenX_LoggingPlugin::RenX_OnDie(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &damageType)
{
	if (damageType.equals("DmgType_Fell")) server->sendLogChan("%s" IRCCOLOR " fell to their death!", RenX::getFormattedPlayerName(player).c_str());
	else if (damageType.equals("DamageType")) server->sendLogChan("%s" IRCCOLOR " switched teams!", RenX::getFormattedPlayerName(player).c_str());
	else server->sendLogChan("%s" IRCCOLOR " died (" IRCCOLOR "12%s" IRCCOLOR ").", RenX::getFormattedPlayerName(player).c_str(), RenX::translateName(damageType.c_str()));
}

void RenX_LoggingPlugin::RenX_OnDestroy(RenX::Server *server, const RenX::PlayerInfo *player, const Jupiter::CStringS &objectName, const Jupiter::CStringS &damageType, RenX::ObjectType type)
{
	if (type == RenX::ObjectType::Building) server->sendLogChan("%s" IRCCOLOR " destroyed the " IRCCOLOR "%s%s" IRCCOLOR " (" IRCCOLOR "12%s" IRCCOLOR ").", RenX::getFormattedPlayerName(player).c_str(), RenX::getTeamColor(RenX::getEnemy(player->team)), RenX::translateName(objectName.c_str()), RenX::translateName(damageType.c_str()));
	else server->sendLogChan("%s" IRCCOLOR " destroyed a " IRCCOLOR "%s%s" IRCCOLOR " (" IRCCOLOR "12%s" IRCCOLOR ").", RenX::getFormattedPlayerName(player).c_str(), RenX::getTeamColor(RenX::getEnemy(player->team)), RenX::translateName(objectName.c_str()), RenX::translateName(damageType.c_str()));
}

void RenX_LoggingPlugin::RenX_OnGameOver(RenX::Server *server, RenX::WinType winType, RenX::TeamType team, int gScore, int nScore)
{
	if (winType == RenX::WinType::Tie)
	{
		if (team == RenX::TeamType::Other) server->sendLogChan(IRCCOLOR "03[Game]" IRCCOLOR "10 The battle ended in a " IRCBOLD "%s" IRCBOLD, RenX::translateWinType(winType));
		else server->sendLogChan(IRCCOLOR "03[Game]" IRCCOLOR "10 The battle ended in a " IRCBOLD "%s" IRCBOLD " - Victory handed to " IRCBOLD IRCCOLOR "%s%s" IRCBOLD, RenX::translateWinType(winType), RenX::getTeamColor(team), RenX::getTeamName(team));
	}
	else server->sendLogChan(IRCCOLOR "03[Game]" IRCCOLOR "%s The " IRCBOLD "%s" IRCBOLD " won by " IRCBOLD "%s" IRCBOLD, RenX::getTeamColor(team), RenX::getFullTeamName(team), RenX::translateWinType(winType));
	server->sendLogChan(IRCCOLOR "03[Game]" IRCCOLOR "%s GDI" IRCCOLOR ": %d | " IRCCOLOR "%sNod" IRCCOLOR ": %d", RenX::getTeamColor(RenX::TeamType::GDI), gScore, RenX::getTeamColor(RenX::TeamType::Nod), nScore);
}

void RenX_LoggingPlugin::RenX_OnGame(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendAdmChan(IRCCOLOR "03[Game]" IRCCOLOR " %s", raw.c_str());
}

void RenX_LoggingPlugin::RenX_OnExecute(RenX::Server *server, const Jupiter::CStringS &user, const Jupiter::CStringS &command)
{
	if (command.matchi("say *")) server->sendLogChan(IRCCOLOR "09EVA: %s", command.c_str() + 4);
	else server->sendAdmChan(IRCCOLOR "07%s executed: %s", user.c_str(), command.c_str());
	// Add a format check later for if user == us.
}

void RenX_LoggingPlugin::RenX_OnSubscribe(RenX::Server *server, const Jupiter::CStringS &user)
{
	server->sendAdmChan(IRCCOLOR "03%s has subscribed to the RCON data stream.", user.c_str());
}

void RenX_LoggingPlugin::RenX_OnRCON(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendAdmChan(IRCCOLOR "05[RCON]" IRCCOLOR " %s", raw.c_str());
}

void RenX_LoggingPlugin::RenX_OnAdminLogin(RenX::Server *server, const RenX::PlayerInfo *player)
{
	server->sendLogChan(IRCCOLOR "07[Admin] " IRCBOLD "%s" IRCBOLD IRCCOLOR " has logged in with " IRCCOLOR "07" IRCBOLD "%s" IRCBOLD IRCNORMAL " privledges.", RenX::getFormattedPlayerName(player).c_str(), player->adminType.c_str());
}

void RenX_LoggingPlugin::RenX_OnAdminLogout(RenX::Server *server, const RenX::PlayerInfo *player)
{
	server->sendAdmChan(IRCCOLOR "07[Admin] " IRCBOLD "%s" IRCBOLD IRCCOLOR " has logged out of their " IRCCOLOR "07" IRCBOLD "%s" IRCBOLD IRCNORMAL " privledges.", RenX::getFormattedPlayerName(player).c_str(), player->adminType.c_str());
}

void RenX_LoggingPlugin::RenX_OnAdmin(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendAdmChan(IRCCOLOR "07[Admin]" IRCCOLOR " %s", raw.c_str());
}

void RenX_LoggingPlugin::RenX_OnLog(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendAdmChan(IRCCOLOR "07[Log]" IRCCOLOR " %s", raw.c_str());
}

void RenX_LoggingPlugin::RenX_OnCommand(RenX::Server *server, const Jupiter::CStringS &raw)
{
	// This is potentially useful later, but not yet!
	//sendAdmChan(server, "[Command] %s", raw.c_str());
}

void RenX_LoggingPlugin::RenX_OnError(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendAdmChan(IRCCOLOR "04[Error]" IRCCOLOR " %s", raw.c_str());
}

void RenX_LoggingPlugin::RenX_OnVersion(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendLogChan(IRCCOLOR "03Renegade X RCON connection established; using RCON verison " IRCBOLD "%d" IRCBOLD " for game version " IRCBOLD "%s" IRCBOLD, atoi(raw.c_str()), raw.c_str() + 3);
}

void RenX_LoggingPlugin::RenX_OnAuthorized(RenX::Server *server, const Jupiter::CStringS &raw)
{
	server->sendLogChan(IRCCOLOR "03RCON authorization completed.");
}

void RenX_LoggingPlugin::RenX_OnOther(RenX::Server *server, char token, const Jupiter::CStringS &raw)
{
	server->sendAdmChan(IRCCOLOR "06[Other]" IRCCOLOR " %s", raw.c_str());
}

// Plugin instantiation and entry point.
RenX_LoggingPlugin pluginInstance;

extern "C" __declspec(dllexport) Jupiter::Plugin *getPlugin()
{
	return &pluginInstance;
}
