/**
 * Copyright (C) 2014 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

#include <cstring> // strlen
#include "SetJoin.h"
#include "IRC_Bot.h"
#include "INIFile.h"
#include "Functions.h"

void SetJoinPlugin::OnJoin(Jupiter::IRC::Client *server, const Jupiter::CStringType &chan, const Jupiter::CStringType &nick)
{
	const char *setjoin = server->Config->get("SetJoins", nick.c_str());
	if (setjoin != nullptr)
	{
		if (setjoin == nullptr) server->sendNotice(nick.c_str(), "No setjoin has been set for you. To set one, use the !setjoin command");
		else
		{
			char *msg = new char[nick.size() + strlen(setjoin) + 32];
			sprintf(msg, IRCBOLD IRCCOLOR "07[%s]" IRCCOLOR IRCBOLD ": %s", nick.c_str(), setjoin);
			server->sendMessage(chan.c_str(), msg);
			delete[] msg;
		}
	}
}

// SetJoin Command

void SetJoinIRCCommand::create()
{
	this->addTrigger("setJoin");
}

void SetJoinIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (parameters.size() > 0)
	{
		const char *timeMsg = getTime();
		source->Config->set("SetJoins", nick.c_str(), parameters.c_str());
		source->Config->sync();
		source->sendMessage(channel.c_str(), "Your setjoin has been set.");
	}
	else source->sendMessage(channel.c_str(), "Too few parameters! Syntax: setJoin <message>");
}

const char *SetJoinIRCCommand::getHelp()
{
	return "Syntax: setJoin <message>";
}

IRC_COMMAND_INIT(SetJoinIRCCommand)

// ViewJoin Command

void ViewJoinIRCCommand::create()
{
	this->addTrigger("viewJoin");
	this->addTrigger("vJoin");
}

void ViewJoinIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	const char *target;
	if (parameters != nullptr) target = parameters.c_str();
	else target = nick.c_str();
	const char *r = source->Config->get("SetJoins", target);
	if (r == nullptr)
	{
		char *msg = new char[strlen(target) + 32];
		sprintf(msg, "No setjoin has been set for \"%s\".", target);
		source->sendMessage(channel.c_str(), msg);
		delete[] msg;
	}
	else
	{
		char *msg = new char[strlen(target) + strlen(r) + 32];
		sprintf(msg, IRCBOLD IRCCOLOR "07[%s]" IRCCOLOR IRCBOLD ": %s", target, r);
		source->sendMessage(channel.c_str(), msg);
		delete[] msg;
	}
}

const char *ViewJoinIRCCommand::getHelp()
{
	return "Syntax: viewJoin [username]";
}

IRC_COMMAND_INIT(ViewJoinIRCCommand)

// DelJoin Command

void DelJoinIRCCommand::create()
{
	this->addTrigger("delJoin");
	this->addTrigger("dJoin");
}

void DelJoinIRCCommand::trigger(IRC_Bot *source, const Jupiter::CStringType &channel, const Jupiter::CStringType &nick, const Jupiter::CStringS &parameters)
{
	if (source->Config->remove("SetJoins", nick.c_str()) > 0) source->sendNotice(nick.c_str(), "Your setjoin has been deleted successfully.");
	else source->sendNotice(nick.c_str(), "No setjoin was found to delete.");
}

const char *DelJoinIRCCommand::getHelp()
{
	return "Syntax: delJoin";
}

IRC_COMMAND_INIT(DelJoinIRCCommand)

// Plugin instantiation and entry point.
SetJoinPlugin pluginInstance;

extern "C" __declspec(dllexport) Jupiter::Plugin *getPlugin()
{
	return &pluginInstance;
}
